#include "InteriorBoundary.h"
#include "Serializer.h"

namespace AMDiS {

  AtomicBoundary& InteriorBoundary::getNewAtomic(int rank)
  {
    boundary[rank].resize(boundary[rank].size() + 1);
    return boundary[rank][boundary[rank].size() - 1];
  }

  void InteriorBoundary::serialize(std::ostream &out)
  {
    int mSize = boundary.size();
    SerUtil::serialize(out, mSize);
    for (RankToBoundMap::iterator it = boundary.begin(); it != boundary.end(); ++it) {
      int rank = it->first;
      int boundSize = it->second.size();
      SerUtil::serialize(out, rank);
      SerUtil::serialize(out, boundSize);
      for (int i = 0; i < boundSize; i++) {
	AtomicBoundary &bound = (it->second)[i];

	SerUtil::serialize(out, bound.rankObject.elIndex);
	SerUtil::serialize(out, bound.rankObject.subObjAtBoundary);
	SerUtil::serialize(out, bound.rankObject.ithObjAtBoundary);

	SerUtil::serialize(out, bound.neighbourObject.elIndex);
	SerUtil::serialize(out, bound.neighbourObject.subObjAtBoundary);
	SerUtil::serialize(out, bound.neighbourObject.ithObjAtBoundary);
      }
    }
  }

  void InteriorBoundary::deserialize(std::istream &in, 
				     std::map<int, Element*> &elIndexMap)
  {
    int mSize = 0;
    SerUtil::deserialize(in, mSize);
    for (int i = 0; i < mSize; i++) {
      int rank = 0;
      int boundSize = 0;
      SerUtil::deserialize(in, rank);
      SerUtil::deserialize(in, boundSize);

      boundary[rank].resize(boundSize);
      for (int i = 0; i < boundSize; i++) {
	AtomicBoundary &bound = boundary[rank][i];

	SerUtil::deserialize(in, bound.rankObject.elIndex);
	SerUtil::deserialize(in, bound.rankObject.subObjAtBoundary);
	SerUtil::deserialize(in, bound.rankObject.ithObjAtBoundary);

	SerUtil::deserialize(in, bound.neighbourObject.elIndex);
	SerUtil::deserialize(in, bound.neighbourObject.subObjAtBoundary);
	SerUtil::deserialize(in, bound.neighbourObject.ithObjAtBoundary);

	bound.rankObject.el = elIndexMap[bound.rankObject.elIndex];
	bound.neighbourObject.el = NULL;
      }
    }
  }
}
