#include "stdlib.h"
#include "AMDiS.h"
#include "MultiGridWrapper.h"

using namespace std;
using namespace AMDiS;

// ===========================================================================
// ===== function definitions ================================================
// ===========================================================================

/// Dirichlet boundary function
class G : public AbstractFunction<double, WorldVector<double> >
{
public:
  /// Implementation of AbstractFunction::operator().
  double operator()(const WorldVector<double>& x) const 
  {
    return exp(-10.0 * (x * x));
  }
};

/// RHS function
class F : public AbstractFunction<double, WorldVector<double> >
{
public:
  F(int degree) : AbstractFunction<double, WorldVector<double> >(degree) {}

  /// Implementation of AbstractFunction::operator().
  double operator()(const WorldVector<double>& x) const 
  {
    int dim = x.getSize();
    double r2 = (x*x);
    double ux = exp(-10.0*r2);
    return -(400.0*r2 - 20.0*dim)*ux;
  }
};


// ===========================================================================
// ===== main program ========================================================
// ===========================================================================
int main(int argc, char* argv[])
{
  FUNCNAME("main");

  // ===== check for init file =====
  TEST_EXIT(argc == 2)("usage: multigrid initfile\n");

  // ===== init parameters =====
  Parameters::init(false, argv[1]);

  // ===== create and init the scalar problem ===== 
  ProblemScal multigrid("multigrid");
  multigrid.initialize(INIT_ALL);

  // === create adapt info ===
  AdaptInfo *adaptInfo = new AdaptInfo("multigrid->adapt");

  // === create adapt ===
  AdaptStationary *adapt = new AdaptStationary("multigrid->adapt",
					       &multigrid,
					       adaptInfo);
  
  //multigrid.setAdapt(adapt);

  // ===== add boundary conditions =====
  multigrid.addDirichletBC(1, new G);

  // ===== create matrix operator =====
  Operator matrixOperator(Operator::MATRIX_OPERATOR, multigrid.getFESpace());
  matrixOperator.addSecondOrderTerm(new Laplace_SOT);
  multigrid.addMatrixOperator(&matrixOperator);

  // ===== create rhs operator =====
  int degree = multigrid.getFESpace()->getBasisFcts()->getDegree();
  Operator rhsOperator(Operator::VECTOR_OPERATOR, multigrid.getFESpace());
  rhsOperator.addZeroOrderTerm(new CoordsAtQP_ZOT(new F(degree)));
  multigrid.addVectorOperator(&rhsOperator);

  adapt->adapt();

  multigrid.writeFiles(adaptInfo, true);
}


